package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLogSystemService;
import cn.estsh.i3plus.core.apiservice.dao.ISysLogSystemDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.platform.bean.SysLogSystem;
import cn.estsh.i3plus.pojo.platform.repositorymongo.SysLogSystemRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreBsonPack;
import io.swagger.annotations.ApiOperation;
import org.bson.conversions.Bson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Description : 系统日志服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 10:41
 * @Modify :
 **/
@Service
public class SysLogSystemService implements ISysLogSystemService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLogSystemService.class);

    @Autowired
    private SysLogSystemRepository logSystemRDao;
    @Autowired
    private ISysLogSystemDao sysLogSystemDao;

    @Override
    @ApiOperation(value = "添加系统日志",notes = "添加系统日志")
    public void insertSysLogSystem(SysLogSystem logSystem) {
        logSystemRDao.insert(logSystem);
    }

    @Override
    @ApiOperation(value = "删除日志",notes = "删除日志")
    public void deleteSysLogSystemById(Long id) {
        logSystemRDao.deleteById(id);
    }

    @Override
    @ApiOperation(value = "查询全部日志",notes = "查询全部日志")
    public List<SysLogSystem> listSysLogSystem() {
        return logSystemRDao.findAll();
    }

    @Override
    @ApiOperation(value = "根据id查询系统日志",notes = "根据id查询系统日志")
    public SysLogSystem getSysLogSystemById(long id) {
        return logSystemRDao.getById(id);
    }

    @Override
    @ApiOperation(value = "系统日志分页复杂查询",notes = "系统日志分页复杂查询")
    public ListPager querySysLogSystemByPager(SysLogSystem logSystem, Pager pager) {
        if (logSystem == null) {
            pager = PagerHelper.getPager(pager,logSystemRDao.listCount());
            return new ListPager(logSystemRDao.listPager(pager),pager);
        } else {
            Bson bson = CoreBsonPack.packBsonByLogSystem(logSystem);
            pager = PagerHelper.getPager(pager,logSystemRDao.findByBsonCount(bson));
            return new ListPager(logSystemRDao.findByBsonPager(bson,pager,logSystem.getOrderByParam()
                                    ,logSystem.getAscOrDesc()),pager);
        }
    }

    @Override
    public Map<String, Object> queryAvgExecuteTime(String startTime, String endTime) {
        return sysLogSystemDao.querySysLogOperateAvgExecuteTime(startTime,endTime);
    }
}
