package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeExpressionService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysTaskTimeExpression;
import cn.estsh.i3plus.pojo.platform.repository.SysTaskTimeExpressionRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysTaskTimeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 定时任务时间表达式 业务逻辑实现
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-24 20:40
 * @Modify :
 **/
@Service
public class SysTaskTimeExpressionService implements ISysTaskTimeExpressionService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskTimeExpressionService.class);

    @Autowired
    private SysTaskTimeExpressionRepository taskTimeExpressionRDao;

    @Autowired
    private SysTaskTimeRepository taskTimeRDao;

    @Override
    @ApiOperation(value = "修改数据")
    public void updateSysTaskTimeExpression(SysTaskTimeExpression taskTimeExpression) {
        LOGGER.info("定时任务时间表达式 TASK_TIME_EXPRESSION :{}", taskTimeExpression);
        taskTimeExpressionRDao.update(taskTimeExpression);
    }

    @Override
    @ApiOperation(value = "根据id删除数据")
    public void deleteSysTaskTimeExpressionById(Long id) {
        LOGGER.info("定时任务时间表达式 TASK_TIME_EXPRESSION :{}", id);
        taskTimeExpressionRDao.deleteById(id);
    }

    @Override
    @ApiOperation(value = "新增数据")
    public SysTaskTimeExpression insertSysTaskTimeExpression(SysTaskTimeExpression taskTimeExpression) {
        LOGGER.info("定时任务时间表达式 TASK_TIME_EXPRESSION :{}", taskTimeExpression);
        return taskTimeExpressionRDao.insert(taskTimeExpression);
    }

    @Override
    @ApiOperation(value = "查询所有数据")
    public List<SysTaskTimeExpression> listSysTaskTimeExpression() {
        LOGGER.info("定时任务时间表达式 TASK_TIME_EXPRESSION list");
        return taskTimeExpressionRDao.list();
    }

    @Override
    @ApiOperation(value = "根据ID 查询任务表达式")
    public SysTaskTimeExpression getSysTaskTimeExpressionById(Long id) {
        LOGGER.info("定时任务时间表达式 TASK_TIME_EXPRESSION :{}" ,id);
        return taskTimeExpressionRDao.getById(id);
    }

    @Override
    @ApiOperation(value = "任务表达式查询，分页，排序")
    public ListPager querySysTaskTimeExpressionByPager(SysTaskTimeExpression taskTimeExpression, Pager pager) {
        LOGGER.info("文件表达式 TASK_TIME_EXPRESSION taskTimeExpression:{}，Pager:{}",taskTimeExpression,pager);
        if(taskTimeExpression == null) {
            pager = PagerHelper.getPager(pager, taskTimeExpressionRDao.listCount());
            return new ListPager(taskTimeExpressionRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysTaskTimeExpression(taskTimeExpression);
            pager = PagerHelper.getPager(pager, taskTimeExpressionRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(taskTimeExpressionRDao.findByHqlWherePage(hqlPack + taskTimeExpression.orderBy(),pager),pager);
        }
    }

    @Override
    @ApiOperation(value = "任务表达式批量删除")
    public void deleteSysTaskTimeExpressionByIds(Long[] ids) {
        LOGGER.info("文件表达式 SYS_MENU ids:{}", ids);
        long count = taskTimeRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("timeExpressionId",ids));
        if(count >= 1){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在关联定时任务无法删除！")
                    .setErrorSolution("请重新操作")
                    .build();
        }else {
            taskTimeExpressionRDao.deleteByIds(ids);
        }
    }
}
