package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserInfoService;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.*;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 用户信息 服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-11-08 19:44
 * @Modify :
 **/
@Service
public class SysUserInfoService implements ISysUserInfoService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserInfoService.class);

    @Autowired
    private SysUserInfoRepository sysUserInfoRDao;

    @Autowired
    private SysUserRepository sysUserRepository;

    @Autowired
    private SysDepartmentRepository departmentRDao;

    @Autowired
    private SysRefUserDepartmentRepository refUserDepartmentRDao;

    @Autowired
    private SysPositionRepository positionRDao;

    @Autowired
    private SysRefUserPositionRepository refUserPositionRDao;

    @Autowired
    private SysOrganizeRepository organizeRDao;

    /***
     * @param userInfo
     * @return
     */
    @Override
    public SysUserInfo insertSysUserInfo(SysUserInfo userInfo) {
        LOGGER.info("用户信息 SysUserInfo userInfo:{}",userInfo);

        return sysUserInfoRDao.insert(userInfo);
    }

    @Override
    public void updateSysUserInfo(SysUserInfo userInfo) {
        LOGGER.info("用户信息 SysUserInfo userInfo:{}",userInfo);
        sysUserInfoRDao.update(userInfo);
    }

    @Override
    public void refreshRefUserDepartment(Long userInfoId, Long[] departmentIds, String userName) {
        LOGGER.info("用户信息 SysUserInfo userInfoId:{} departmentIds:{} userName:{}",userInfoId,departmentIds,userName);

        SysUserInfo userInfo = sysUserInfoRDao.getById(userInfoId);
        if(userInfo != null ){
            // 查询部门信息
            StringBuffer where = new StringBuffer();
            HqlPack.getInPack(StringUtils.join(departmentIds,","),"id",where);
            List<SysDepartment> departmentList = departmentRDao.findByHqlWhere(where.toString());

            // 删除用户部门关系
            refUserDepartmentRDao.deleteByProperty("userId",userInfo.getId());

            if(departmentList != null && departmentList.size() >0){
                List<SysRefUserDepartment> list = new ArrayList<>();
                List<String> nameList = new ArrayList<>();
                SysRefUserDepartment ref = null;
                for (SysDepartment department : departmentList) {
                    ref = new SysRefUserDepartment();
                    ref.setDepartmentId(department.getId());
                    ref.setDepartmentNameRdd(department.getName());
                    ref.setUserId(userInfo.getId());
                    ref.setUserNameRdd(userInfo.getName());
                    ref.setCreateUser(userName);
                    ref.setCreateDatetime(TimeTool.getNowTime(true));
                    list.add(ref);

                    nameList.add(department.getName());
                }

                userInfo.setDepartmentNamesRdd(String.join(",",nameList));

                refUserDepartmentRDao.saveAll(list);
                sysUserInfoRDao.save(userInfo);

                LOGGER.info("用户信息 SysUserInfo RefUserDepartmentRDao Insert Size :{}",list.size());
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefUserPosition(Long userInfoId, Long[] positionIds , String userName) {
        LOGGER.info("用户信息 SysUserInfo userInfoId:{} positionIds:{} userName:{}",userInfoId,positionIds,userName);

        SysUserInfo userInfo = sysUserInfoRDao.getById(userInfoId);
        if(userInfo != null ){
            // 查询部门信息
            StringBuffer where = new StringBuffer();
            HqlPack.getInPack(StringUtils.join(positionIds,","),"id",where);
            List<SysPosition> positionList = positionRDao.findByHqlWhere(where.toString());

            // 删除用户岗位关系
            refUserPositionRDao.deleteByProperty("userId",userInfo.getId());

            if(positionList != null && positionList.size() >0){
                List<SysRefUserPosition> list = new ArrayList<>();
                List<String> nameList = new ArrayList<>();
                SysRefUserPosition ref = null;
                for (SysPosition position : positionList) {
                    ref = new SysRefUserPosition();
                    ref.setUserId(userInfo.getId());
                    ref.setUserNameRdd(userInfo.getName());
                    ref.setPositionId(position.getId());
                    ref.setPositionNameRdd(position.getName());
                    ref.setCreateUser(userName);
                    ref.setCreateDatetime(TimeTool.getNowTime(true));
                    list.add(ref);

                    nameList.add(position.getName());
                }

                userInfo.setPositionNamesRdd(String.join(",",nameList));

                refUserPositionRDao.saveAll(list);
                sysUserInfoRDao.save(userInfo);

                LOGGER.info("用户信息 SysUserInfo RefUserPositionRDao Insert Size :{}",list.size());
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefUserOrganize(Long userInfoId, Long organizeId, String userName) {
        LOGGER.info("用户信息 SysUserInfo userInfoId:{} organizeId:{} userName:{}",userInfoId,organizeId,userName);
        SysUserInfo userInfo = sysUserInfoRDao.getById(userInfoId);
        if(userInfo != null ){
            SysOrganize organize = organizeRDao.getById(organizeId);
            if(organize != null ){

                userInfo.setOrganizeId(organize.getId());
                userInfo.setOrganizeNameRdd(organize.getName());
                userInfo.setModifyUser(userName);
                userInfo.setModifyDatetime(TimeTool.getNowTime(true));

                sysUserInfoRDao.update(organize);
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void deleteSysUserInfoById(Long id) {
        LOGGER.info("用户信息 SysUserInfo id:{}",id);
        sysUserInfoRDao.deleteById(id);
    }

    @Override
    public void deleteSysUserInfoByIds(Long[] ids) {
        LOGGER.info("用户信息 SysUserInfo ids:{}",ids);
        sysUserInfoRDao.deleteByIds(ids);
    }

    @Override
    public SysUserInfo getSysUserInfoById(Long id) {
        return sysUserInfoRDao.getById(id);
    }

    @Override
    public List listSysUserInfo() {
        return sysUserInfoRDao.findAll();
    }

    @Override
    public ListPager<SysUserInfo> querySysUserInfo(SysUserInfo userInfo, Pager pager) {
        LOGGER.info("定时任务 TASK_TIME TaskTime:{}，Pager:{}",userInfo,pager);
        if(userInfo == null) {
            pager = PagerHelper.getPager(pager, sysUserInfoRDao.listCount());
            return new ListPager(sysUserInfoRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysUserInfo(userInfo);
            pager = PagerHelper.getPager(pager, sysUserInfoRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysUserInfoRDao.findByHqlWherePage(hqlPack + userInfo.orderBy(),pager),pager);
        }
    }

    @Override
    @ApiOperation(value = "关系查询-用户部门",notes = "查询用户部门关系")
    public List<SysRefUserDepartment> findRefUserDepartmentByUserId(Long userId) {
        return refUserDepartmentRDao.findByProperty("userId",userId);
    }

    @Override
    @ApiOperation(value = "关系查询-用户岗位",notes = "查询用户岗位关系")
    public List<SysRefUserPosition> findSysRefUserPositionByUserId(Long userId) {
        return refUserPositionRDao.findByProperty("userId",userId);
    }

    @Override
    @ApiOperation(value = "用户唯一校验",notes = "校验用户信息唯一性")
    public void checkSysUserInfoOnly(SysUserInfo userInfo) {
        if(userInfo != null){
            // 登录名唯一教研
            if(!checkUserInfoByUserEmpNo(userInfo.getUserEmpNo(), userInfo.getId() == null ? 0 : userInfo.getId())){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                        .setErrorDetail("工号已存在")
                        .setErrorSolution("请重新输入工号")
                        .build();
            }
        }
    }

    @Override
    @ApiOperation(value = "用户唯一校验  用户工号",notes = "用户唯一校验  用户工号")
    public boolean checkUserInfoByUserEmpNo(String no, long id) {
        int count;
        if(id > 0){
            StringBuffer sw = new StringBuffer();
            HqlPack.getStringEqualPack(no,"userEmpNo",sw);
            HqlPack.getNumNOEqualPack(id,"id",sw);
            count = sysUserInfoRDao.findByHqlWhereCount(sw.toString());
        }else{
            count = sysUserInfoRDao.findByPropertyCount("userEmpNo",no);
        }
        return count <= 0 ;
    }
}
